@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -53,13 +53,14 @@
 #include <QtGui/qevent.h>
 #include <QtCore/qlocale.h>
 #include <QtGui/qfont.h>
+#include <QtGui/qpalette.h>
 #include <QtQuickTemplates2/private/qtquicktemplates2global_p.h>
 #include <QtQml/qqml.h>
 #include <QtQml/qqmllist.h>
 #include <QtQml/qqmlparserstatus.h>
 #include <QtQuick/qquickitem.h>
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 #include <QtGui/qaccessible.h>
 #endif
 
@@ -84,7 +85,6 @@ class Q_QUICKTEMPLATES2_PRIVATE_EXPORT QQuickPopup : public QObject, public QQml
     Q_PROPERTY(qreal contentHeight READ contentHeight WRITE setContentHeight NOTIFY contentHeightChanged FINAL)
     Q_PROPERTY(qreal availableWidth READ availableWidth NOTIFY availableWidthChanged FINAL)
     Q_PROPERTY(qreal availableHeight READ availableHeight NOTIFY availableHeightChanged FINAL)
-    Q_PROPERTY(qreal spacing READ spacing WRITE setSpacing RESET resetSpacing NOTIFY spacingChanged FINAL REVISION 1)
     Q_PROPERTY(qreal margins READ margins WRITE setMargins RESET resetMargins NOTIFY marginsChanged FINAL)
     Q_PROPERTY(qreal topMargin READ topMargin WRITE setTopMargin RESET resetTopMargin NOTIFY topMarginChanged FINAL)
     Q_PROPERTY(qreal leftMargin READ leftMargin WRITE setLeftMargin RESET resetLeftMargin NOTIFY leftMarginChanged FINAL)
@@ -97,7 +97,7 @@ class Q_QUICKTEMPLATES2_PRIVATE_EXPORT QQuickPopup : public QObject, public QQml
     Q_PROPERTY(qreal bottomPadding READ bottomPadding WRITE setBottomPadding RESET resetBottomPadding NOTIFY bottomPaddingChanged FINAL)
     Q_PROPERTY(QLocale locale READ locale WRITE setLocale RESET resetLocale NOTIFY localeChanged FINAL)
     Q_PROPERTY(QFont font READ font WRITE setFont RESET resetFont NOTIFY fontChanged FINAL)
-    Q_PROPERTY(QQuickItem *parent READ parentItem WRITE setParentItem NOTIFY parentChanged FINAL)
+    Q_PROPERTY(QQuickItem *parent READ parentItem WRITE setParentItem RESET resetParentItem NOTIFY parentChanged FINAL)
     Q_PROPERTY(QQuickItem *background READ background WRITE setBackground NOTIFY backgroundChanged FINAL)
     Q_PROPERTY(QQuickItem *contentItem READ contentItem WRITE setContentItem NOTIFY contentItemChanged FINAL)
     Q_PROPERTY(QQmlListProperty<QObject> contentData READ contentData FINAL)
@@ -110,10 +110,17 @@ class Q_QUICKTEMPLATES2_PRIVATE_EXPORT QQuickPopup : public QObject, public QQml
     Q_PROPERTY(bool visible READ isVisible WRITE setVisible NOTIFY visibleChanged FINAL)
     Q_PROPERTY(qreal opacity READ opacity WRITE setOpacity NOTIFY opacityChanged FINAL)
     Q_PROPERTY(qreal scale READ scale WRITE setScale NOTIFY scaleChanged FINAL)
-    Q_PROPERTY(ClosePolicy closePolicy READ closePolicy WRITE setClosePolicy NOTIFY closePolicyChanged FINAL)
-    Q_PROPERTY(TransformOrigin transformOrigin READ transformOrigin WRITE setTransformOrigin)
+    Q_PROPERTY(ClosePolicy closePolicy READ closePolicy WRITE setClosePolicy RESET resetClosePolicy NOTIFY closePolicyChanged FINAL)
+    Q_PROPERTY(TransformOrigin transformOrigin READ transformOrigin WRITE setTransformOrigin FINAL)
     Q_PROPERTY(QQuickTransition *enter READ enter WRITE setEnter NOTIFY enterChanged FINAL)
     Q_PROPERTY(QQuickTransition *exit READ exit WRITE setExit NOTIFY exitChanged FINAL)
+    // 2.1 (Qt 5.8)
+    Q_PROPERTY(qreal spacing READ spacing WRITE setSpacing RESET resetSpacing NOTIFY spacingChanged FINAL REVISION 1)
+    // 2.3 (Qt 5.10)
+    Q_PROPERTY(bool opened READ isOpened NOTIFY openedChanged FINAL REVISION 3)
+    Q_PROPERTY(bool mirrored READ isMirrored NOTIFY mirroredChanged FINAL REVISION 3)
+    Q_PROPERTY(bool enabled READ isEnabled WRITE setEnabled NOTIFY enabledChanged FINAL REVISION 3)
+    Q_PROPERTY(QPalette palette READ palette WRITE setPalette RESET resetPalette NOTIFY paletteChanged FINAL REVISION 3)
     Q_CLASSINFO("DefaultProperty", "contentData")
 
 public:
@@ -155,10 +162,6 @@ public:
     qreal availableWidth() const;
     qreal availableHeight() const;
 
-    qreal spacing() const;
-    void setSpacing(qreal spacing);
-    void resetSpacing();
-
     qreal margins() const;
     void setMargins(qreal margins);
     void resetMargins();
@@ -212,6 +215,7 @@ public:
 
     QQuickItem *parentItem() const;
     void setParentItem(QQuickItem *parent);
+    void resetParentItem();
 
     QQuickItem *background() const;
     void setBackground(QQuickItem *background);
@@ -259,6 +263,7 @@ public:
 
     ClosePolicy closePolicy() const;
     void setClosePolicy(ClosePolicy policy);
+    void resetClosePolicy();
 
     // keep in sync with Item.TransformOrigin
     enum TransformOrigin {
@@ -282,11 +287,31 @@ public:
 
     Q_INVOKABLE void forceActiveFocus(Qt::FocusReason reason = Qt::OtherFocusReason);
 
+    // 2.1 (Qt 5.8)
+    qreal spacing() const;
+    void setSpacing(qreal spacing);
+    void resetSpacing();
+
+    // 2.3 (Qt 5.10)
+    bool isOpened() const;
+    bool isMirrored() const;
+
+    bool isEnabled() const;
+    void setEnabled(bool enabled);
+
+    QPalette palette() const;
+    void setPalette(const QPalette &palette);
+    void resetPalette();
+
 public Q_SLOTS:
     void open();
     void close();
 
 Q_SIGNALS:
+    void opened();
+    void closed();
+    void aboutToShow();
+    void aboutToHide();
     void xChanged();
     void yChanged();
     void zChanged();
@@ -298,7 +323,6 @@ Q_SIGNALS:
     void contentHeightChanged();
     void availableWidthChanged();
     void availableHeightChanged();
-    Q_REVISION(1) void spacingChanged();
     void marginsChanged();
     void topMarginChanged();
     void leftMarginChanged();
@@ -327,11 +351,13 @@ Q_SIGNALS:
     void enterChanged();
     void exitChanged();
     void windowChanged(QQuickWindow *window);
-
-    void aboutToShow();
-    void aboutToHide();
-    void opened();
-    void closed();
+    // 2.1 (Qt 5.8)
+    Q_REVISION(1) void spacingChanged();
+    // 2.3 (Qt 5.10)
+    Q_REVISION(3) void openedChanged();
+    Q_REVISION(3) void mirroredChanged();
+    Q_REVISION(3) void enabledChanged();
+    Q_REVISION(3) void paletteChanged();
 
 protected:
     QQuickPopup(QQuickPopupPrivate &dd, QObject *parent);
@@ -351,7 +377,13 @@ protected:
     virtual void mouseDoubleClickEvent(QMouseEvent *event);
     virtual void mouseUngrabEvent();
     virtual bool overlayEvent(QQuickItem *item, QEvent *event);
+#if QT_CONFIG(quicktemplates2_multitouch)
+    virtual void touchEvent(QTouchEvent *event);
+    virtual void touchUngrabEvent();
+#endif
+#if QT_CONFIG(wheelevent)
     virtual void wheelEvent(QWheelEvent *event);
+#endif
 
     virtual void contentItemChange(QQuickItem *newItem, QQuickItem *oldItem);
     virtual void fontChange(const QFont &newFont, const QFont &oldFont);
@@ -360,11 +392,13 @@ protected:
     virtual void itemChange(QQuickItem::ItemChange change, const QQuickItem::ItemChangeData &data);
     virtual void marginsChange(const QMarginsF &newMargins, const QMarginsF &oldMargins);
     virtual void paddingChange(const QMarginsF &newPadding, const QMarginsF &oldPadding);
+    virtual void paletteChange(const QPalette &newPalette, const QPalette &oldPalette);
     virtual void spacingChange(qreal newSpacing, qreal oldSpacing);
 
     virtual QFont defaultFont() const;
+    virtual QPalette defaultPalette() const;
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
     virtual QAccessible::Role accessibleRole() const;
     virtual void accessibilityActiveChanged(bool active);
 #endif
@@ -380,6 +414,7 @@ private:
     Q_DECLARE_PRIVATE(QQuickPopup)
     friend class QQuickPopupItem;
     friend class QQuickOverlay;
+    friend class QQuickOverlayPrivate;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QQuickPopup::ClosePolicy)
