@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -48,6 +48,7 @@
 // We mean it.
 //
 
+#include <QtGui/qpalette.h>
 #include <QtQuick/private/qquicktextedit_p.h>
 #include <QtQuickTemplates2/private/qtquicktemplates2global_p.h>
 
@@ -67,6 +68,11 @@ class Q_QUICKTEMPLATES2_PRIVATE_EXPORT QQuickTextArea : public QQuickTextEdit
     Q_PROPERTY(QQuickItem *background READ background WRITE setBackground NOTIFY backgroundChanged FINAL)
     Q_PROPERTY(QString placeholderText READ placeholderText WRITE setPlaceholderText NOTIFY placeholderTextChanged FINAL)
     Q_PROPERTY(Qt::FocusReason focusReason READ focusReason WRITE setFocusReason NOTIFY focusReasonChanged FINAL)
+    // 2.1 (Qt 5.8)
+    Q_PROPERTY(bool hovered READ isHovered NOTIFY hoveredChanged FINAL REVISION 1)
+    Q_PROPERTY(bool hoverEnabled READ isHoverEnabled WRITE setHoverEnabled RESET resetHoverEnabled NOTIFY hoverEnabledChanged FINAL REVISION 1)
+    // 2.3 (Qt 5.10)
+    Q_PROPERTY(QPalette palette READ palette WRITE setPalette RESET resetPalette NOTIFY paletteChanged FINAL REVISION 3)
 
 public:
     explicit QQuickTextArea(QQuickItem *parent = nullptr);
@@ -88,6 +94,19 @@ public:
 
     bool contains(const QPointF &point) const override;
 
+    // 2.1 (Qt 5.8)
+    bool isHovered() const;
+    void setHovered(bool hovered);
+
+    bool isHoverEnabled() const;
+    void setHoverEnabled(bool enabled);
+    void resetHoverEnabled();
+
+    // 2.3 (Qt 5.10)
+    QPalette palette() const;
+    void setPalette(const QPalette &palette);
+    void resetPalette();
+
 Q_SIGNALS:
     void fontChanged();
     void implicitWidthChanged3();
@@ -96,6 +115,13 @@ Q_SIGNALS:
     void placeholderTextChanged();
     void focusReasonChanged();
     void pressAndHold(QQuickMouseEvent *event);
+    // 2.1 (Qt 5.8)
+    Q_REVISION(1) void pressed(QQuickMouseEvent *event);
+    Q_REVISION(1) void released(QQuickMouseEvent *event);
+    Q_REVISION(1) void hoveredChanged();
+    Q_REVISION(1) void hoverEnabledChanged();
+    // 2.3 (Qt 5.10)
+    Q_REVISION(3) void paletteChanged();
 
 protected:
     void classBegin() override;
@@ -107,6 +133,10 @@ protected:
 
     void focusInEvent(QFocusEvent *event) override;
     void focusOutEvent(QFocusEvent *event) override;
+#if QT_CONFIG(quicktemplates2_hover)
+    void hoverEnterEvent(QHoverEvent *event) override;
+    void hoverLeaveEvent(QHoverEvent *event) override;
+#endif
     void mousePressEvent(QMouseEvent *event) override;
     void mouseMoveEvent(QMouseEvent *event) override;
     void mouseReleaseEvent(QMouseEvent *event) override;
@@ -127,7 +157,6 @@ class Q_QUICKTEMPLATES2_PRIVATE_EXPORT QQuickTextAreaAttached : public QObject
 
 public:
     explicit QQuickTextAreaAttached(QObject *parent);
-    ~QQuickTextAreaAttached();
 
     QQuickTextArea *flickable() const;
     void setFlickable(QQuickTextArea *control);
